/* 
* Paquet ordres
* conté els mètodes: alta, modificació, cancel.lació i execució d'ordres de compra/venda
*
*/

spool ./pkg_ordres.log;
--capçalera paquet
create or replace package pkg_ordre is

--procediment per crear una nova ordre de compra/venda d'accions
procedure prc_ordre_alta (
	pIdClient NUMBER, 
	pIdAccio NUMBER,
	pQuantitat NUMBER, 
	pTipus varchar2,
	pOrigen varchar2,
	pPreuObjectiu number default 0,
	rsp OUT varchar2
	);

--procediment per modificar els paràmetres quantitat i preuOperacio de ordres en curs, en situació pendent
-- passant el paràmetre id
procedure prc_ordre_modificacio (
	pIdOrdre NUMBER,
	pQuantitat NUMBER, 
	pPreuObjectiu number,
	rsp OUT varchar2
	);

--procediment per cancel.lar una ordre en curs, en situació pendent
-- passant el paràmetre id
procedure prc_ordre_cancelacio (
	pIdOrdre NUMBER,
	rsp OUT varchar2
	);

--procediment per executar una ordre en curs, en situació pendent
-- passant el paràmetre id
procedure prc_ordre_execucio (
	pIdOrdre NUMBER,
	rsp OUT varchar2
	);

--procediment per executar automàticament les ordres que compleixin la condició de preu objectiu
-- si estan en situació pendent passant el el Id de l'acció i el seu nou preu
procedure prc_ordre_execucio_auto (
	pIdAccio NUMBER,
	pValorVigent NUMBER, 
	rsp OUT varchar2
	);

-- procediment per executar automàticament les ordres que compleixin la condició de preuObjectiu
--
procedure prc_ordre_execucio_auto (
	rsp OUT varchar2	
	);

end pkg_ordre;
/


-- cos del paquet
CREATE OR REPLACE PACKAGE BODY pkg_ordre is

/*
* definicio del procediment per crear una nova ordre de compra/venda d'accions
* es passa els paràmetres de l'identificador del client, de l'acció, la quantitat a adjudicar,
* del tipus d'ordre, de l'origen, i opcionalment el preu objectiu de l'operació
*
*/
procedure prc_ordre_alta (
	pIdClient NUMBER, 
	pIdAccio NUMBER,
	pQuantitat NUMBER, 
	pTipus varchar2,
	pOrigen varchar2,
	pPreuObjectiu number default 0,
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ORDRE_ALTA';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	numOrdresPD number(4);
	var_quantitat cartera.quantitat%type;
	var_accEsActiu accions.esActiu%type;
	var_cliEsActiu clients.esActiu%type;
	ordreExistentPendent EXCEPTION; -- es defineix una excepció pròpia
	massesAccionsPerVendre EXCEPTION;
	AccioInactiva EXCEPTION;
BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdClient||', '||pIdAccio||', '||pQuantitat||', '||pTipus||', '||pOrigen||', '||pPreuObjectiu;
--<<fi del recull dels paràmetres d'entrada



	if pIdClient is null or pIdAccio is null or pQuantitat is null or pTipus is null or pOrigen is null then
		raise admin_tfc.pkg_control.insert_null;
	end if;

	-- es verifica que l'acció és activa
		select esActiu
			into var_accEsActiu
			from accions
			where idaccio = pIdAccio;
	-- es verifica que el client és actiu
		select esActiu
			into var_cliEsActiu
			from clients
			where idclient = pIdClient;

-- les noves ordres no poden estar vinculades a clients o accions no actives
		if var_accEsActiu = 'N' or var_cliEsActiu = 'N' then
			raise AccioInactiva;
		end if;

--verifica que no hi ha donada d'alta cap ordre del mateix client i per la mateixa ordre en situació pendent
	select count(*) 
	INTO numOrdresPD
	from ordres
		where estat = 'PD' and client = pIdClient and accio = pIdAccio;

	if numOrdresPD = 0 then

--s'ha de tancar la consulta dins el seu propi bloc per gestionar l'excepcio de NO_DATA_FOUND
		BEGIN
			select quantitat
			into var_quantitat
			from cartera 
			where client = pIdClient and accio = pIdAccio;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				var_quantitat := 0;
		END;

		
-- no es poden posar a la venda més accions que les disponibles en cartera
		if pTipus = 'VENDA' and pQuantitat > var_quantitat then
			raise massesAccionsPerVendre;
		end if;


--inserció a la taula ORDRES del nou registre amb les dades passades
		INSERT INTO ordres (idordre, client, accio, quantitat, tipus, origen, preuobjectiu, dataentrada, estat)
			VALUES (ordres_seq.nextval, pIdClient, pIdAccio, pQuantitat, pTipus, pOrigen, pPreuObjectiu, sysdate, 'PD');

-- si la nova ordre és de VENDA s'insereix registre que actualitza la taula estadística TF_accionsDisponibles
		IF pTipus = 'VENDA' THEN
			pkg_dwh.prc_ins_tfAccDisponibles(pIdAccio);
		END IF;

--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
		rsp := 'OK';
	else
		raise ordreExistentPendent;
	end if;

	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN admin_tfc.pkg_control.insert_null THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;
		
		WHEN AccioInactiva THEN
			sError := pkg_control.fun_error(6)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN massesAccionsPerVendre THEN
			sError := pkg_control.fun_error(5)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN ordreExistentPendent THEN
			sError := pkg_control.fun_error(7)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;
		
		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_ordre_alta;

/*
* definicio del procediment per modificar els paràmetres autoritzats de una ordre de compra/venda d'accions
* només es poden modificar les ordres que estan en situació pendent (PD)
* es passa els paràmetres de l'identificador de l'ordre , la quantitat i el preu objectiu
*
*/
procedure prc_ordre_modificacio (
	pIdOrdre NUMBER, 
	pQuantitat NUMBER, 
	pPreuObjectiu NUMBER,
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ORDRE_MODIFICACIO';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	var_ordres ordres%rowtype;
	situacioNoValida EXCEPTION; -- es defineix una excepció pròpia
	massesAccionsPerVendre EXCEPTION;
	var_quantitat NUMBER(4);

BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdOrdre||', '||pQuantitat||', '||pPreuObjectiu;
--<<fi del recull dels paràmetres d'entrada

	
	if pIdOrdre is null then
		raise admin_tfc.pkg_control.insert_null;
	end if;

	SELECT * 
		INTO var_ordres
		from ordres o
		where o.idordre = pIdOrdre;

-- no es poden posar a la venda més accions que les disponibles
	BEGIN
		select quantitat
		into var_quantitat
		from cartera 
		where client = var_ordres.client and accio = var_ordres.accio;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			var_quantitat := 0;
	END;


	if var_ordres.estat = 'PD' then
--es sel.lecciona l'ordre amb el id passat per paràmetre
-- i es modifica el registre  de la taula d'ordres amb les noves dades passades
-- es verifica que no s'intentin posar a la venda més accions que les disponibles

		
	--si el paràmetre d'entrada no és null s'actualitza el registre, si és null es manté el valor anterior
		var_ordres.quantitat := NVL(pQuantitat, var_ordres.quantitat);
		var_ordres.preuobjectiu := NVL (pPreuObjectiu, var_ordres.preuobjectiu);

-- no es poden posar a la venda més accions que les disponibles en cartera
		IF var_ordres.tipus = 'VENDA' and pQuantitat > var_quantitat THEN
			raise massesAccionsPerVendre;
		END IF;

--actualització de la taula ordres amb els nous valors	
		UPDATE ordres 
			set row = var_ordres 
			where idordre = pIdOrdre;

-- si la ordre modificada és de VENDA s'actualitza la taula estadística TF_accionsDisponibles
		IF var_ordres.tipus = 'VENDA' THEN
			pkg_dwh.prc_ins_tfAccDisponibles(var_ordres.accio);
		END IF;	

--registra a la taula Log l'execució del procediment
		pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
		rsp := 'OK';
	else
		raise situacioNoValida;
	end if;
	
	
	EXCEPTION
		WHEN massesAccionsPerVendre THEN
			sError := pkg_control.fun_error(5)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;

		when situacioNoValida then
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		WHEN DUP_VAL_ON_INDEX THEN
			sError := pkg_control.fun_error(3)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_ordre_modificacio;
	

/*
* definicio del procediment per cancel.lar una ordre de compra/venda d'accions
* només es poden cancel.lar les ordres que estan en situació pendent (PD)
* es passa el paràmetres de l'identificador de l'ordre 
*
*/
procedure prc_ordre_cancelacio (
	pIdOrdre NUMBER,
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ORDRE_CANCELACIO';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	reg_ordre ordres%rowtype;
	situacioNoValida EXCEPTION; -- es defineix una excepció pròpia
BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdOrdre;
--<<fi del recull dels paràmetres d'entrada
	
	if pIdOrdre is null then
		raise admin_tfc.pkg_control.insert_null;
	end if;

	select *
	into reg_ordre
	from ordres o
	where o.idordre = pIdOrdre;
	
	if reg_ordre.estat = 'PD' then
		UPDATE ORDRES
			SET ESTAT ='CA'
			where idordre = pIdOrdre;
		
	ELSE
		raise situacioNoValida;
	end if;

-- si l'ordre era de VENDA, s'actualitza la taula TF_accions_disponibles del mòdul estadístic
	IF reg_ordre.tipus = 'VENDA' THEN
		pkg_dwh.prc_ins_tfAccDisponibles(reg_ordre.accio);
	END IF;

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';
	
	EXCEPTION
		when situacioNoValida then
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;
		
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_ordre_cancelacio;

/*
* definicio del procediment per executar una ordre de compra/venda d'accions
* només es poden executar les ordres que estan en situació pendent (PD)
* es passa el paràmetres de l'identificador de l'ordre 
*
*/
procedure prc_ordre_execucio (
	pIdOrdre NUMBER,
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ORDRE_EXECUCIO';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	var_ordres ordres%rowtype;
	var_preu accions.valorvigent%type;
	situacioNoValida EXCEPTION; -- es defineix una excepció pròpia
BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdOrdre;
--<<fi del recull dels paràmetres d'entrada

	
	if pIdOrdre is null then
		raise admin_tfc.pkg_control.insert_null;
	end if;

	select *
	into var_ordres
	from ordres
	where idordre = pIdOrdre;

	select valorvigent
	into var_preu
	from accions
	where idaccio = var_ordres.accio;

	if var_ordres.estat != 'PD' then
		raise situacioNoValida;
	end if;
	
	if var_ordres.tipus = 'COMPRA' then

		pkg_cartera.prc_cartera_adjudicar(var_ordres.client, var_ordres.accio, var_ordres.quantitat, sError);

		UPDATE ORDRES
			SET ESTAT ='EX', dataexecucio = sysdate, preuObtingut = var_preu, total = var_ordres.quantitat*var_preu
			where idordre = pIdOrdre;
		
	ELSIF var_ordres.tipus = 'VENDA' then

		pkg_cartera.prc_cartera_retirar(var_ordres.client, var_ordres.accio, var_ordres.quantitat, sError);

		UPDATE ORDRES
			SET ESTAT ='EX', dataexecucio = sysdate, preuObtingut = var_preu, total = var_ordres.quantitat*var_preu
			where idordre = pIdOrdre;

-- si l'ordre era de VENDA, s'actualitza la taula TF_accions disponibles del mòdul estadístic
		pkg_dwh.prc_ins_tfAccDisponibles(var_ordres.accio);

	end if;

-- s'actualitza la taula TF_acciomesNegociada del mòdul estadístic
	pkg_dwh.prc_ins_tfAccMesNegociada;

-- s'actualitza la taula TF_carteraValor del mòdul estadístic
	pkg_dwh.prc_ins_tfCarteraValors;

-- s'actualitza la taula TF_beneficis del mòdul estadístic
	pkg_dwh.prc_ins_tfBeneficis;

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';
	
			
	EXCEPTION
		when situacioNoValida then
			sError := pkg_control.fun_error(9)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;
			
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;
		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_ordre_execucio;


/*
* definicio del procediment per executar les ordres que compleixen la condició de preu objectiu
* només es poden executar les ordres que estan en situació pendent (PD)
* es passa el id de l'acció i el seu nou preu
*
*/
procedure prc_ordre_execucio_auto (
	pIdAccio NUMBER,
	pValorVigent NUMBER, 
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ORDRE_EXECUCIO_AUTO';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	capOrdre EXCEPTION; -- es defineix una excepció pròpia
	comptador number (4) := 0;

	cursor cursorLlistaOrdres (pcIdAccio NUMBER)
	IS
		select o1.idordre, o1.accio, o1.client, o1.preuObjectiu, o1.tipus, o1.quantitat
			from ordres o1
      		join clients cl
      		on cl.idclient = o1.client
			where o1.estat = 'PD' and cl.esactiu = 'Y' and o1.accio = pcIdAccio
			for update of o1.estat, o1.dataexecucio, o1.preuobtingut;


BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := pIdAccio||', '||pValorVigent;
--<<fi del recull dels paràmetres d'entrada

	FOR var_ordres IN cursorLlistaOrdres (pIdAccio)
	LOOP
		if (var_ordres.tipus = 'COMPRA' AND 
			(var_ordres.preuObjectiu>=pValorVigent or var_ordres.preuObjectiu=0)) THEN
			
			pkg_cartera.prc_cartera_adjudicar(var_ordres.client, var_ordres.accio, var_ordres.quantitat, sError);
			UPDATE ORDRES
			SET ESTAT ='EX', dataexecucio = sysdate, preuObtingut = pValorVigent, total = var_ordres.quantitat*pValorVigent
			where CURRENT OF cursorLlistaOrdres;
			
		
		elsif (var_ordres.tipus = 'VENDA' AND 
			(var_ordres.preuObjectiu<=pValorVigent or var_ordres.preuObjectiu=0)) THEN
			
			pkg_cartera.prc_cartera_retirar(var_ordres.client, var_ordres.accio, var_ordres.quantitat, sError);
			UPDATE ORDRES
				SET ESTAT ='EX', dataexecucio = sysdate, preuObtingut = pValorVigent, total = var_ordres.quantitat*pValorVigent
				where CURRENT OF cursorLlistaOrdres;

		-- si l'ordre es de VENDA, s'actualitza la taula TF_accions disponibles del mòdul estadístic
			pkg_dwh.prc_ins_tfAccDisponibles(var_ordres.accio);
			
		end if;
		
		comptador := comptador + 1;	
	END LOOP;

	if comptador = 0 THEN
		raise capOrdre;
	END IF;

-- s'actualitza la taula TF_acciomesNegociada del mòdul estadístic
	pkg_dwh.prc_ins_tfAccMesNegociada;

-- s'actualitza la taula TF_carteraValor del mòdul estadístic
	pkg_dwh.prc_ins_tfCarteraValors;

-- s'actualitza la taula TF_beneficis del mòdul estadístic
	pkg_dwh.prc_ins_tfBeneficis;

--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK');
--s'informa del paràmetre de sortida
	rsp := 'OK';
	
			
	EXCEPTION
		WHEN capOrdre THEN
			if cursorLlistaOrdres%isopen THEN
				close cursorLlistaOrdres;
			end if;
			
			sError := pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;
			
		WHEN NO_DATA_FOUND THEN
			sError := pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;
		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_ordre_execucio_auto;



/*
* definicio del procediment per executar AUTOMÀTICAMENT les ordres de compra/venda d'accions
* que compleixen amb el criteri de preuObjectiu
*
*/
procedure prc_ordre_execucio_auto (
	rsp OUT varchar2
	) IS
-- declaració de variables locals del procediment
	prc_nom varchar2(25 CHAR):='PRC_ORDRE_EXECUCIO_AUTO';
	prc_paramIN varchar(100 CHAR) := null;
	sError varchar(512);
	comptador number (4) := 0;
	var_valor_accio accions.valorvigent%type;
	capOrdre EXCEPTION; -- es defineix una excepció pròpia

	cursor cursorLlistaOrdres
	IS
		select idordre, accio, client, preuObjectiu, tipus, quantitat
			from ordres
			where estat = 'PD'
			for update of estat, dataexecucio, preuobtingut;

	cursor cursorAccions(pIdaccio accions.idaccio%type)
	IS 
		select valorvigent
			from accions
			where idaccio = pIdaccio;


BEGIN

-->>recull dels paràmetres d'entrada a un string
	prc_paramIN := 'null';
--<<fi del recull dels paràmetres d'entrada

	FOR var_ordres IN cursorLlistaOrdres 
	LOOP
		OPEN cursorAccions(var_ordres.accio);
		fetch cursorAccions into var_valor_accio;
		DBMS_OUTPUT.PUT_LINE('auto '||var_ordres.idordre||', '||var_valor_accio );

		if (var_ordres.tipus = 'COMPRA' AND 
			(var_ordres.preuObjectiu>=var_valor_accio or var_ordres.preuObjectiu=0)) THEN
			
			pkg_cartera.prc_cartera_adjudicar(var_ordres.client, var_ordres.accio, var_ordres.quantitat, sError);
			--DBMS_OUTPUT.PUT_LINE('exec auto' || var_ordres.preuObjectiu||', '||var_valor_accio);
			UPDATE ORDRES
			SET ESTAT ='EX', dataexecucio = sysdate, preuObtingut = var_valor_accio, total = var_ordres.quantitat*var_valor_accio
			where CURRENT OF cursorLlistaOrdres;
			
		
		elsif (var_ordres.tipus = 'VENDA' AND 
			(var_ordres.preuObjectiu<=var_valor_accio or var_ordres.preuObjectiu=0)) THEN
			
			pkg_cartera.prc_cartera_retirar(var_ordres.client, var_ordres.accio, var_ordres.quantitat, sError);
			UPDATE ORDRES
				SET ESTAT ='EX', dataexecucio = sysdate, preuObtingut = var_valor_accio, total = var_ordres.quantitat*var_valor_accio
				where CURRENT OF cursorLlistaOrdres;

-- si l'ordre es de VENDA, s'actualitza la taula TF_accions disponibles del mòdul estadístic
			pkg_dwh.prc_ins_tfAccDisponibles(var_ordres.accio);
			
		end if;
		close cursorAccions;
		comptador := comptador + 1;
	END LOOP;

	if comptador = 0 THEN
		raise capOrdre;
	END IF;

-- s'actualitza la taula TF_acciomesNegociada del mòdul estadístic
	pkg_dwh.prc_ins_tfAccMesNegociada;

-- s'actualitza la taula TF_carteraValor del mòdul estadístic
	pkg_dwh.prc_ins_tfCarteraValors;

-- s'actualitza la taula TF_beneficis del mòdul estadístic
	pkg_dwh.prc_ins_tfBeneficis;


--registra a la taula Log l'execució del procediment
	pkg_control.prc_logProcediment(prc_nom, prc_paramIN, 'FI OK: '||comptador||' registres modificats');
--s'informa del paràmetre de sortida
	rsp := 'OK';

	EXCEPTION
		
		WHEN capOrdre THEN
			if cursorLlistaOrdres%isopen THEN
				close cursorLlistaOrdres;
			end if;
			
			sError :=  pkg_control.fun_error(4)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN NO_DATA_FOUND THEN
			sError :=  pkg_control.fun_error(2)||substr(SQLERRM, 1, 500);
	--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
	-- s'informa del paràmetre de sortida
			rsp := sError;

		WHEN others THEN
			sError := pkg_control.fun_error(1)||substr(SQLERRM, 1, 500);
--registra a la taula Log l'execució del procediment
			pkg_control.prc_logProcediment(prc_nom, prc_paramIN, sError);
-- s'informa del paràmetre de sortida
			rsp := sError;

END prc_ordre_execucio_auto;

-- fi del paquet
end pkg_ordre;
/
spool off;